#!/bin/sh

# Blosh - um compilador de blogs e websites estáticos
# Copyright (C) 2014,2015 Adriano Rafael Gomes <adrianorg@arg.eti.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

main() {
    load_configuration
    init_working_area
    init_output_dir

    sort_posts_by_date
    group_posts_by_tag
    group_posts_by_date

    create_html_posts_pages   # posts/*.html
    create_html_index \
        $WORKING_AREA_DIR/posts \
        $OUTPUT_DIR/index.html \
        "Últimos artigos publicados" \
        $BASE_URL/index.rss   # index.html
    create_rss_index \
        $WORKING_AREA_DIR/posts \
        $OUTPUT_DIR/index.rss \
        "Últimos artigos publicados" \
        $BASE_URL/index.html  # index.rss

    create_html_tags_pages # tags/*/index.html
    create_rss_tags_pages  # tags/*/index.rss
    create_html_date_pages # date/*/index.html
    create_html_pages      # pages/path/to/*.html

    copy_static_files
}

load_configuration() {
    if [ -r blosh.conf ]
    then
        . ./blosh.conf
        BASE_URL=$URL_DOMAIN/$URL_ROOT
        vars="URL_DOMAIN URL_ROOT BASE_URL BLOG_TITLE WORKING_AREA_DIR OUTPUT_DIR"
        echo $vars | tr " " "\n" | while read k
        do
            v=$(eval echo \$$k)
            if [ -z "$v" ]
            then
                echo "configuração obrigatória $k não definida"
                exit 1
            fi
        done
    else
        echo "arquivo de configuração blosh.conf não encontrado"
        exit 1
    fi
}

init_working_area() {
    rm -r $WORKING_AREA_DIR/posts
    rm -r $WORKING_AREA_DIR/tags
    rm -r $WORKING_AREA_DIR/date
    mkdir -p $WORKING_AREA_DIR/posts
    mkdir -p $WORKING_AREA_DIR/tags
    mkdir -p $WORKING_AREA_DIR/date
}

init_output_dir() {
    rm -r $OUTPUT_DIR/posts
    rm -r $OUTPUT_DIR/tags
    rm -r $OUTPUT_DIR/date
    rm -r $OUTPUT_DIR/pages
    mkdir -p $OUTPUT_DIR/posts
    mkdir -p $OUTPUT_DIR/tags
    mkdir -p $OUTPUT_DIR/date
    mkdir -p $OUTPUT_DIR/pages
}

sort_posts_by_date() {
    for f in $(ls -1 ./posts/*.markdown)
    do
        echo $f
        date=$(get_post_date $f)
        time=$(date -d "$date" "+%s")
        permalink=$(get_post_permalink $f)
        ln -s ../../$f $WORKING_AREA_DIR/posts/$time-$permalink.markdown
    done
}

group_posts_by_tag() {
    for f in $(ls -1 ./posts/*.markdown)
    do
        echo $f
        date=$(get_post_date $f)
        time=$(date -d "$date" "+%s")
        permalink=$(get_post_permalink $f)
        tags=$(get_post_tags $f)
        if [ -n "$tags" ]
        then
            echo "$tags" | tr " " "\n" | while read tag
            do
                normalized_tag=$(normalize_string $tag)
                mkdir -p $WORKING_AREA_DIR/tags/$normalized_tag
                ln -s ../../../$f $WORKING_AREA_DIR/tags/$normalized_tag/$time-$permalink.markdown
            done
        fi
    done
}

group_posts_by_date() {
    for f in $(ls -1 ./posts/*.markdown)
    do
        echo $f
        date=$(get_post_date $f)
        time=$(date -d "$date" "+%s")
        year_month=$(date -d "$date" "+%Y-%m")
        permalink=$(get_post_permalink $f)
        mkdir -p $WORKING_AREA_DIR/date/$year_month
        ln -s ../../../$f $WORKING_AREA_DIR/date/$year_month/$time-$permalink.markdown
    done
}

create_html_posts_pages() {
    for f in $(ls -1r $WORKING_AREA_DIR/posts/*.markdown)
    do
        echo $f
        permalink=$(get_post_permalink $f)
        print_html_post_page $f | tidy_html > $OUTPUT_DIR/posts/$permalink.html
    done
}

print_html_post_page() {
    post="$1"

    subtitle=$(get_post_title $post)
    print_html_header "$subtitle" "" "" "show_sidebar"
    print_html_post_body $post "full"
    print_html_footer "show_sidebar"
}

create_html_index() {
    input_dir="$1"
    output_file="$2"
    subtitle="$3"
    rss_file="$4"

    print_html_index "$input_dir" "$subtitle" "$rss_file" | tidy_html > $output_file
}

print_html_index() {
    input_dir="$1"
    subtitle="$2"
    rss_file="$3"

    print_html_header "$subtitle" "$rss_file" "show_subtitle" "show_sidebar"
    for f in $(ls -1r $input_dir/*.markdown)
    do
        print_html_post_body $f "partial"
    done
    print_html_footer "show_sidebar"
}

create_html_tags_pages() {
    for d in $(ls -1 $WORKING_AREA_DIR/tags)
    do
        echo $d
        mkdir -p $OUTPUT_DIR/tags/$d
        create_html_index \
            $WORKING_AREA_DIR/tags/$d \
            $OUTPUT_DIR/tags/$d/index.html \
            "Artigos publicados com a tag $d" \
            $BASE_URL/tags/$d/index.rss
    done
}

create_rss_tags_pages() {
    for d in $(ls -1 $WORKING_AREA_DIR/tags)
    do
        echo $d
        mkdir -p $OUTPUT_DIR/tags/$d
        create_rss_index \
            $WORKING_AREA_DIR/tags/$d \
            $OUTPUT_DIR/tags/$d/index.rss \
            "Artigos publicados com a tag $d" \
            $BASE_URL/tags/$d/index.html
    done
}

print_html_tags_list() {
    echo "<h2>Tags</h2>"
    echo "<ul>"
    for d in $(ls -1 $WORKING_AREA_DIR/tags)
    do
        count=$(ls -1 $WORKING_AREA_DIR/tags/$d | wc -l)
        echo "<li><a href="/$URL_ROOT/tags/$d">$d</a> ($count)</li>"
    done
    echo "</ul>"
}

create_html_date_pages() {
    for d in $(ls -1 $WORKING_AREA_DIR/date)
    do
        echo $d
        mkdir -p $OUTPUT_DIR/date/$d
        create_html_index \
            $WORKING_AREA_DIR/date/$d \
            $OUTPUT_DIR/date/$d/index.html \
            "Artigos publicados em $d" \
            ""
    done
}

print_html_date_list() {
    echo "<h2>Data</h2>"
    echo "<ul>"
    for d in $(ls -1r $WORKING_AREA_DIR/date)
    do
        count=$(ls -1 $WORKING_AREA_DIR/date/$d | wc -l)
        echo "<li><a href="/$URL_ROOT/date/$d">$d</a> ($count)</li>"
    done
    echo "</ul>"
}

print_html_header() {
    subtitle="$1"
    rss_file="$2"
    show_subtitle="$3"
    show_sidebar="$4"

    title="$BLOG_TITLE$(if [ -n "$subtitle" ]; then echo " - $subtitle"; fi)"
    cat <<EOF
    <html>
        <head>
            <link rel="stylesheet" href="$BASE_URL/style.css" type="text/css">
            <link rel="icon" href="$BASE_URL/favicon.png" type="image/png">
            $(if [ -n "$rss_file" ]; then echo "<link rel=\"alternate\" href=\"$rss_file\" type=\"application/rss+xml\" title=\"RSS\">"; fi)
            <title>$title</title>
        </head>
        <body>
            <div id="header">
                <h1>$BLOG_TITLE</h1>
                <p>$BLOG_DESCRIPTION</p>
                $(print_html_menu "$rss_file")
                $(if [ "$show_subtitle" = "show_subtitle" ]
                then
                    echo "<p><strong>$subtitle</strong></p>"
                else
                    echo "<p><strong>&nbsp;</strong></p>"
                fi)
            </div>
            $(if [ "$show_sidebar" = "show_sidebar" ]
            then
                echo "<div id=\"content-posts\">"
            else
                echo "<div id=\"content-pages\">"
            fi)
EOF
}

print_html_menu() {
    rss_file="$1"

    cat <<EOF
    <p>
    <a href="/$URL_ROOT">Início</a>
EOF
    echo "$MENU_PAGES" | tr " " "\n" | while read menu_page
    do
        page="./pages/$menu_page.markdown"
        dir=$(dirname $page)
        title=$(get_post_title $page)
        link="/$URL_ROOT/$dir/$(get_page_permalink $page).html"
        echo "<a href=\"$link\">$title</a>"
    done
    if [ -n "$rss_file" ]
    then
        echo "<a href=\"$rss_file\">Siga-me via RSS</a>"
    fi
    echo "</p>"
}

print_html_footer() {
    show_sidebar="$1"
    cat <<EOF
            </div>
EOF
    if [ "$show_sidebar" = "show_sidebar" ]
    then
        cat <<EOF
            <div id="sidebar">
                $(print_html_tags_list)
                $(print_html_date_list)
            </div>
EOF
    fi
    cat <<EOF
            <div id="footer" class="page-footer">
                <p>$BLOG_FOOTNOTE</p>
            </div>
        </body>
    </html>
EOF
}

print_html_post_body() {
    post="$1"
    partial_or_full="$2"

    title=$(get_post_title $post)
    date=$(get_post_date $post)
    tags=$(get_post_tags $post)
    link="/$URL_ROOT/posts/$(get_post_permalink $post).html"
    if [ "$partial_or_full" = "partial" ]
    then
        body=$(get_post_body_partial $post $link)
    else
        body=$(get_post_body $post)
    fi
    cat <<EOF
    <div class="post">
        <div class="post-header">
            <h2>$title</h2>
        </div>
        <div class="post-body">$body</div>
        <div class="post-footer">
            <p>
                <a href="$link">Artigo completo</a>
                | Publicado em $date
                $(if [ -n "$tags" ]; then echo " | Tags $(print_html_tags "$tags")"; fi)
            </p>
        </div>
    </div>
EOF
}

print_html_tags() {
    echo "$tags" | tr " " "\n" | while read tag
    do
        cat <<EOF
        <a href="/$URL_ROOT/tags/$(normalize_string $tag)">$tag</a>
EOF
    done
}

create_rss_index() {
    input_dir="$1"
    output_file="$2"
    subtitle="$3"
    html_file="$4"

    print_rss_index "$input_dir" "$subtitle" "$html_file" | tidy_rss > $output_file
}

print_rss_index() {
    input_dir="$1"
    subtitle="$2"
    html_file="$3"

    subtitle="$BLOG_TITLE$(if [ -n "$subtitle" ]; then echo " - $subtitle"; fi)"
    cat <<EOF
    <?xml version="1.0"?>
    <rss version="2.0">
        <channel>
            <title>$subtitle</title>
            <link>$html_file</link>
            <description>$BLOG_DESCRIPTION</description>
EOF

    for f in $(ls -1r $input_dir/*.markdown)
    do
        print_rss_post $f "partial"
    done

    cat <<EOF
        </channel>
    </rss>
EOF
}

print_rss_post() {
    post="$1"
    partial_or_full="$2"

    title=$(get_post_title $post)
    date=$(get_post_date $post)
    tags=$(get_post_tags $post)
    link="$BASE_URL/posts/$(get_post_permalink $post).html"
    if [ "$partial_or_full" = "partial" ]
    then
        body=$(get_post_body_partial $post $link)
    else
        body=$(get_post_body $post)
    fi
    cat <<EOF
    <item>
        <title>$title</title>
        <pubDate>$(date -d "$date" -R)</pubDate>
        <link>$link</link>
        <description>$(echo $body | escape_to_xml_entities)</description>
    </item>
EOF
}

get_post_header() {
    grep "^#\s*$1\s*:" -i -m 1 $2 | cut -d : -f 2- | sed -e "s/^\s*//"
}

get_post_title() {
    get_post_header "title" $1
}

get_post_date() {
    get_post_header "date" $1
}

get_post_tags() {
    get_post_header "tags" $1
}

get_post_url() {
    get_post_header "url" $1
}

get_page_permalink() {
    post="$1"

    url=$(get_post_url $post)
    if [ -z "$url" ]
    then
        title=$(get_post_title $post)
        url=$(normalize_string "$title")
    fi
    echo $url
}

get_post_permalink() {
    post="$1"

    url=$(get_page_permalink $post)
    date=$(get_post_date $post)
    echo "$(date -d "$date" "+%F")-$url"
}

get_post_body() {
    cat $1 | sed -e "1,/^$/d" | markdown
}

get_post_body_partial() {
    post="$1"
    link="$2"

    escaped_link=$(echo "$link" | sed -e 's/\//\\\//g')
    get_post_body $post |
        sed -e "/<!-- break -->/q" |
        sed -e "s/<!-- break -->/\n[Continue lendo \&raquo;]($escaped_link)\n/" |
        markdown
}

normalize_string() {
    echo $1 | tr [:upper:] [:lower:] | tr -d [:punct:] | tr [:blank:] - | tr -s - | iconv -f UTF-8 -t ASCII//TRANSLIT
}

tidy_html() {
    tidy -quiet --show-warnings false -indent -wrap 160 --input-encoding utf8
}

tidy_rss() {
    tidy -quiet --show-warnings false -indent -wrap 160 --input-encoding utf8 -xml
}

escape_to_xml_entities() {
    sed -e 's/&/\&amp;/g' \
        -e 's/"/\&quot;/g' \
        -e "s/'/\&apos;/g" \
        -e 's/</\&lt;/g' \
        -e 's/>/\&gt;/g'
}

create_html_pages() {
    for f in $(find ./pages/ -name "*.markdown")
    do
        echo $f
        dir=$(dirname $f)
        permalink=$(get_page_permalink $f)
        mkdir -p $OUTPUT_DIR/$dir
        print_html_page $f | tidy_html > $OUTPUT_DIR/$dir/$permalink.html
    done
}

print_html_page() {
    post="$1"

    subtitle=$(get_post_title $post)
    print_html_header "$subtitle" "" "" ""
    print_html_page_body $post
    print_html_footer ""
}

print_html_page_body() {
    page="$1"

    title=$(get_post_title $page)
    date=$(get_post_date $page)
    link="/$URL_ROOT/pages/$(get_page_permalink $page).html"
    body=$(get_post_body $page)
    cat <<EOF
    <div class="post">
        <div class="post-header">
            <h2>$title</h2>
        </div>
        <div class="post-body">$body</div>
        <div class="post-footer">
            <p>
                Página atualizada em $date
            </p>
        </div>
    </div>
EOF
}

copy_static_files() {
    cp -a ./static/* $OUTPUT_DIR
}

main
